local addon_name, addon = ...
local core = addon.core

local util = addon.utility
local time = addon.time
local menu = addon.menu
local global = addon.global
local _G = _G

local player = UnitName("player")
local _, class = UnitClass("player")
local realm = GetRealmName()

local SECOND = 1
local MINUTE = 60 * SECOND
local HOUR = 60 * MINUTE
local DAY = 24 * HOUR
local WEEK = 7 * DAY

local function VersionChanges() --TODO: Switch to a registeration setup?
   if not _G.WOWMMGlobal[realm] then
      _G.WOWMMGlobal[realm] = _G.WOWMMGlobal[realm] or {}
   end
   -- Changing the format of the save table from previous versions.
   -- I do not want to lose previously saved data.

   -- Version information added in 1.0.6.
   -- All version information before 1.0.6 would be nil.
   local version = _G.WOWMMGlobal.Version or "1.0.5.1"
   local realmVersion = _G.WOWMMGlobal[realm] and _G.WOWMMGlobal[realm].Version or "1.0.5.1"
   local charVersion = _G.WOWMMChar.Version or "1.0.5.1"

   -- Format change from version 1.0.5.1 to 1.0.6 for introduction of currency information.
   if _G.WOWMMGlobal[realm].Chars and realmVersion <= "1.0.5.1" then
      local OldFormat = {}
      for k, v in pairs(_G.WOWMMGlobal[realm].Chars) do
         if type(v) == "number" then
            OldFormat[k] = v
         end
      end

      for k, v in pairs(OldFormat) do
         _G.WOWMMGlobal[realm].Chars[k] = {}
         _G.WOWMMGlobal[realm].Chars[k].Cash = v
      end
   end

   -- Changing MovingAvg to just Avg which is more appropriate.
   if charVersion <= "1.0.7.1" then
      local prevSetting = _G.WOWMMChar.MovingAvgShown or true
      _G.WOWMMChar.AvgShown = prevSetting
      _G.WOWMMChar.MovingAvgShown = nil
   end

   -- Changed the way I determine a weekly reset has occurred
   if _G.WOWMMGlobal[realm].Week and realmVersion <= "1.0.7.1" then
      local wd = time.GetCalendarDate(time.GetDaysToAdvance())
      local id = (wd - 3) % 7
      local lastLoginTime =  _G.WOWMMGlobal[realm].Week.LLIT or 0
      local logonTime = time.GetTimeSinceEpoch(time.GetDaysToAdvance())
      if (logonTime - lastLoginTime) < WEEK and id >= _G.WOWMMGlobal[realm].Week.ID then
         _G.WOWMMGlobal[realm].Week.ID = time.GetTimeOfLastReset(time.GetDaysToAdvance())
      else
         _G.WOWMMGlobal[realm].Week.ID = 0
      end
      _G.WOWMMGlobal[realm].Week.LLIT = nil
   end

   -- Do this last to make sure we don't overwrite the previous version
   local cVersion = GetAddOnMetadata(addon_name, "version")
   _G.WOWMMGlobal.Version = cVersion
   _G.WOWMMGlobal[realm].Version = cVersion
   _G.WOWMMChar.Version = cVersion
end

local function SetupCurrentPlayer() 
   local cash = GetMoney()
   _G.WOWMMGlobal[realm].Chars[player] = _G.WOWMMGlobal[realm].Chars[player] or {}
   _G.WOWMMGlobal[realm].Chars[player].Cash = _G.WOWMMGlobal[realm].Chars[player].Cash or cash
end

local function InitializePersistantVariables(newSession)
   if newSession == nil then
      newSession = true
   end

   VersionChanges()

   _G.WOWMMGlobal[realm].Chars = _G.WOWMMGlobal[realm].Chars or {}
   _G.WOWMMGlobal[realm].Classes = _G.WOWMMGlobal[realm].Classes or {}
   _G.WOWMMGlobal[realm].Classes[player] = class
   
   local IDSet = time.GetIDSet()
   for k, v in pairs(IDSet) do
      if k ~= "Session" or newSession then
         local invalidID = false
         _G.WOWMMGlobal[realm][k] = _G.WOWMMGlobal[realm][k] or {}
         local prevEarned = _G.WOWMMGlobal[realm][k].Earned or 0
         local prevSpent = _G.WOWMMGlobal[realm][k].Spent or 0
         if v ~= "none" then
            _G.WOWMMGlobal[realm][k].ID = _G.WOWMMGlobal[realm][k].ID or v
            if _G.WOWMMGlobal[realm][k].ID == v then
               _G.WOWMMGlobal[realm][k].Earned = _G.WOWMMGlobal[realm][k].Earned or 0
               _G.WOWMMGlobal[realm][k].Spent = _G.WOWMMGlobal[realm][k].Spent or 0
            else
               _G.WOWMMGlobal[realm][k].ID = v
               _G.WOWMMGlobal[realm][k].Earned = 0
               _G.WOWMMGlobal[realm][k].Spent = 0
               invalidID = true
            end
         elseif k == "Session" then
            _G.WOWMMGlobal[realm][k].Earned = 0
            _G.WOWMMGlobal[realm][k].Spent = 0
            invalidID = true
         else -- AllTime
            _G.WOWMMGlobal[realm][k].Earned = _G.WOWMMGlobal[realm][k].Earned or 0
            _G.WOWMMGlobal[realm][k].Spent = _G.WOWMMGlobal[realm][k].Spent or 0
         end
         
         -- Average Information
         local netIndex = k.."Net"
         local countIndex = k.."Count"
         if util.getSetValue(time.TimeSetAvg, k) then
            local net = prevEarned - prevSpent
            _G.WOWMMGlobal[realm][netIndex] = _G.WOWMMGlobal[realm][netIndex] or net
            _G.WOWMMGlobal[realm][countIndex] = _G.WOWMMGlobal[realm][countIndex] or 1

            if invalidID then
               _G.WOWMMGlobal[realm][netIndex] = _G.WOWMMGlobal[realm][netIndex] + net
               _G.WOWMMGlobal[realm][countIndex] = _G.WOWMMGlobal[realm][countIndex] + 1
            end
         end

         --[===[@debug
         -- Currency Information
         local NumCurrencies = GetCurrencyListSize()
         _G.WOWMMChar.Currency = _G.WOWMMChar.Currency or {}
         _G.WOWMMChar.Currency[k] = _G.WOWMMChar.Currency[k] or {}
         for i = 1, NumCurrencies do
            local currency, isHeader, _, isUnused = GetCurrencyListInfo(i)
            if not isHeader and not isUnused then
               _G.WOWMMChar.Currency[k][currency] = _G.WOWMMChar.Currency[k][currency] or {} 
               _G.WOWMMChar.Currency[k][currency].Spent = _G.WOWMMChar.Currency[k][currency].Spent or 0
               _G.WOWMMChar.Currency[k][currency].Earned = _G.WOWMMChar.Currency[k][currency].Earned or 0
            end
         end
         if invalidID then
            for i = 1, NumCurrencies do
               local currency, isHeader, _, isUnused = GetCurrencyListInfo(i)
               if not isHeader and not isUnused then
                  _G.WOWMMChar.Currency[k][currency] = {}
                  _G.WOWMMChar.Currency[k][currency].Spent = 0
                  _G.WOWMMChar.Currency[k][currency].Earned = 0
               end
            end
         end
         --@end-debug]===]
      end
   end

   SetupCurrentPlayer()
   
   _G.WOWMMGlobal[realm].Total = util.GetRealmTotalCash(realm)
end

local function UpdatePersistantVariables()
   local cash = GetMoney()

   if not _G.WOWMMGlobal[realm].Chars[player] then
      SetupCurrentPlayer()
   end
   pcash = _G.WOWMMGlobal[realm].Chars[player].Cash
   
   util.UpdateEarnedSpent(cash, pcash)
   
   _G.WOWMMGlobal[realm].Chars[player].Cash = cash
   _G.WOWMMGlobal[realm].Classes[player] = class
   _G.WOWMMGlobal[realm].Total = util.GetRealmTotalCash(realm)

end

local loaded = false
local frame = CreateFrame("Frame")
local function EventHandler(self, event, ...)
   local args = {...}
   addon.debugPrint(event, ...)

   if event == "ADDON_LOADED" and args[1] ~= addon_name then
      return
   end

   if event == "ADDON_LOADED" and args[1] == addon_name then
      global.InitGlobals()
   end

   if event =="PLAYER_ENTERING_WORLD" then
      if not loaded then
         InitializePersistantVariables()
         loaded = true
      end
      UpdatePersistantVariables()
   elseif event == "PLAYER_MONEY" then
      UpdatePersistantVariables()
   end
end
util.RegisterEvents(frame, EventHandler,
                                    'PLAYER_ENTERING_WORLD',
                                    'ADDON_LOADED',
                                    'PLAYER_MONEY')

core.InitializePersistantVariables = InitializePersistantVariables
